#!/usr/bin/env python

# Python Standard Modules

# MUGQIC Modules
from core.config import *
from core.job import *

def index(input):
    return Job(
        [input],
        [input + ".bai"],
        [['samtools_index', 'module_samtools']],
        command="""\
samtools index \\
  {input}""".format(
        input=input
        )
    )

def faidx(input):
    return Job(
        [input],
        [input + ".fai"],
        [['samtools_index', 'module_samtools']],
        command="""\
samtools faidx \\
  {input}""".format(
        input=input
        )
    )

def flagstat(input, output):
    return Job(
        [input],
        [output],
        [['samtools_flagstat', 'module_samtools']],
        command="""\
samtools flagstat \\
  {input} \\
  > {output}""".format(
        input=input,
        output=output
        )
    )

def mpileup(input_bams, output, other_options="", region=None):
    return Job(
        input_bams,
        [output],
        [['samtools_mpileup', 'module_samtools']],
        command="""\
samtools mpileup {other_options} \\
  -f {reference_fasta}{region}{input_bams}{output}""".format(
        other_options=other_options,
        reference_fasta=config.param('samtools_mpileup', 'genome_fasta', type='filepath'),
        region=" \\\n  -r " + region if region else "",
        input_bams="".join([" \\\n  " + input_bam for input_bam in input_bams]),
        output=" \\\n  > " + output if output else ""
        )
    )

def sort(input_bam, output_prefix, sort_by_name=False):
    output_bam = output_prefix + ".bam"

    return Job(
        [input_bam],
        [output_bam],
        [['samtools_sort', 'module_samtools']],
        command="""\
samtools sort {other_options}{sort_by_name} \\
  {input_bam} \\
  {output_prefix}""".format(
        other_options=config.param('samtools_sort', 'other_options', required=False),
        sort_by_name=" -n" if sort_by_name else "",
        input_bam=input_bam,
        output_prefix=output_prefix
        ),
        removable_files=[output_bam]
    )

def view(input, output=None, options=""):
    return Job(
        [input],
        [output],
        [['samtools_view', 'module_samtools']],
        command="""\
samtools view {options} \\
  {input}{output}""".format(
        options=options,
        input=input,
        output=" \\\n  > " + output if output else ""
        ),
        removable_files=[output]
    )

def bcftools_cat(inputs, output):
    return Job(
        inputs,
        [output],
        [['bcftools_cat', 'module_samtools']],
        command="""\
bcftools cat \\
  {inputs}{output}""".format(
        inputs=" \\\n  ".join(inputs),
        output=" \\\n  > " + output if output else ""
        )
    )

def bcftools_view(input, output, options="", pair_calling=False):
    return Job(
        [input],
        [output],
        [['bcftools_view', 'module_samtools']],
        command="""\
bcftools view {pair_calling} {options} \\
  {input}{output}""".format(
        options=options,
        pair_calling="-T pair" if pair_calling else "",
        input=input,
        output=" \\\n  > " + output if output else ""
        )
    )
